/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 20, 2001.
   -
   - The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
   - Portions created by Henk-Johan van Rantwijk are Copyright (C) 2001
   - Henk-Johan van Rantwijk.  All Rights Reserved.
   -
   - Contributor(s):
   -  Henk-Johan van Rantwijk <bugs4hj@netscape.net> (Original Author)
   -
   - Alternatively, the contents of this file may be used under the
   - terms of the GNU General Public License Version 2 or later (the
   - "GPL"), in which case the provisions of the GPL are applicable
   - instead of those above.  If you wish to allow use of your
   - version of this file only under the terms of the GPL and not to
   - allow others to use your version of this file under the MPL,
   - indicate your decision by deleting the provisions above and
   - replace them with the notice and other provisions required by
   - the GPL.  If you do not delete the provisions above, a recipient
   - may use your version of this file under either the MPL or the
   - GPL.
   -
*/
//////////////////////////////////////////////////////////////////////////
// Returns nsiURI. This is used in function initializeRDFStorage()
function getProfileURI()
{
  var profile = Components.classes["@mozilla.org/profile/manager;1"].getService(Components.interfaces.nsIProfileInternal);
  var gIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  var URI = gIOService.newFileURI(profile.getProfileDir(profile.currentProfile));
  return URI || null;
}
////////////////////////////////////////////////////////////////////////////////
// This initializes or build a new RDF
function initializeRDFStorageFile()
{
  var rdfFile = "multizilla.rdf"; // "quickprefs.rdf" for stand-alone versions of QPM
  // Get profile directory of current profile
  var profileURI = getProfileURI();
  // Construct file handler to our RDF storage file
  const ioService = Components.classes['@mozilla.org/network/io-service;1']
                              .getService(Components.interfaces.nsIIOService);
  var gFileHandler = ioService.getProtocolHandler("file")
                              .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
  var storageFile = gFileHandler.getFileFromURLSpec(profileURI.spec + rdfFile)
                                .QueryInterface(Components.interfaces.nsILocalFile);
  // Check if "multizilla.rdf" exists
  if (!storageFile.exists()) {
    /* The RDF is missing, we need to construct one ourself as fallback
       Note: this has partly been ported from C++ (nsLocalStore.cpp#410) */
    const defaultRDF = "<?xml version=\"1.0\"?>\n" +
                       "<RDF:RDF xmlns:RDF=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n" +
                       "         xmlns:MZ=\"http://multizilla.mozdev.org/rdf#\">\n" +
                       "  <!-- Empty -->\n" +
                       "</RDF:RDF>\n";
    /* Create a new RDF
       See also: http://lxr.mozilla.org/seamonkey/source/netwerk/base/public/nsIFileStreams.idl
            and: http://lxr.mozilla.org/seamonkey/source/nsprpub/pr/include/prio.h */
	storageFile.create(storageFile.NORMAL_FILE_TYPE, 0666);
    // Setup a stream
    var stream = Components.classes['@mozilla.org/network/file-output-stream;1']
                           .createInstance(Components.interfaces.nsIFileOutputStream);
    // Initialize the new RDF
    stream.init(storageFile, 2, 0x200, false);
    // Write the XML data into our RDF
    stream.write(defaultRDF, defaultRDF.length);
    // Close the stream
    stream.close();
  }
  // We should have a RDF by now. Initialize global 'mzRDFStore'
  mzRDFStore = RDF.GetDataSource(profileURI.spec + rdfFile);
}
////////////////////////////////////////////////////////////////////////////////
function constructContainer(aType, aRecource)
{
  var RDFCUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
  var tabContainer = Components.classes['@mozilla.org/rdf/container;1'].createInstance(Components.interfaces.nsIRDFContainer);

  switch(aType) 
  {
    default:
    case "seq": RDFCUtils.MakeSeq(mzRDFStore, RDF.GetResource(aRecource));
      break;
    case "bag": RDFCUtils.MakeBag(mzRDFStore, RDF.GetResource(aRecource));
      break;
    case "alt": RDFCUtils.MakeAlt(mzRDFStore, RDF.GetResource(aRecource));
      break;
  }
  tabContainer.Init(mzRDFStore, RDF.GetResource(aRecource));
  return tabContainer;
}
////////////////////////////////////////////////////////////////////////////////
function saveBrowserSession()
{
  var sessionID = 0;
  var windowID = 0;
  /***
  * This can easily be expanded to open all tabs of all windows, but that
  * will have to be introduced in a next update because of lack of time (:
  */
  storeGroupData("urn:session-"+sessionID+":window-"+windowID+":tabs");
}
////////////////////////////////////////////////////////////////////////////////
function storeGroupData(aResource)
{
  var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
  var tab = gBrowser.mTabList;
  removeAllResourcesFor(aResource + ":root"); // "urn:session-x:window-x:tabs" + ":root"
  var tabContainer = constructContainer("seq", aResource + ":root");
  var tabs = gBrowser.mTabContainer.childNodes.length;

  for (var index = 0; index < tabs; index++) {
    var label = tab[index].getAttribute("label");
    var customLabel = tab[index].getAttribute("customLabel") ? tab[index].getAttribute("label") : "false";
    var image = tab[index].getAttribute("image");
    var browser = gBrowser.getBrowserForTab(tab[index]);
    var location = browser.webNavigation.currentURI.spec;
    var interval = tab[index].mReloadInterval;
    var ordinal = tab[index].ordinal;

    var plugins = browser.docShell.allowPlugins.valueOf();
    var javascript = browser.docShell.allowJavascript.valueOf();
    var metaredirects = browser.docShell.allowMetaRedirects.valueOf();
    var subframes = browser.docShell.allowSubframes.valueOf();
    var images = browser.docShell.allowImages.valueOf();

    var selected = (tab[index] == gBrowser.selectedTab) ? "true" : "false";

    var newNode = RDF.GetResource(aResource + ":tab-" + index); // "urn:session-x:window-x:tabs" + ":tab-x"
    var MZ ="http://multizilla.mozdev.org/rdf#";
    var labelProp = RDF.GetResource(MZ+"label");
    var customProp = RDF.GetResource(MZ+"custom");
    var imageProp = RDF.GetResource(MZ+"image");
    var locationProp = RDF.GetResource(MZ+"location");
    var intervalProp = RDF.GetResource(MZ+"interval");
    var ordinalProp = RDF.GetResource(MZ+"ordinal");

    var pluginsProp = RDF.GetResource(MZ+"plugins");
    var javascriptProp = RDF.GetResource(MZ+"javascript");
    var metaredirectsProp = RDF.GetResource(MZ+"metaredirects");
    var subframesProp = RDF.GetResource(MZ+"subframes");
    var imagesProp = RDF.GetResource(MZ+"images");

    var selectedProp = RDF.GetResource(MZ+"selected");

    mzRDFStore.Assert(newNode, labelProp, RDF.GetLiteral(label), true);
    mzRDFStore.Assert(newNode, customProp, RDF.GetLiteral(customLabel), true);
    mzRDFStore.Assert(newNode, imageProp, RDF.GetLiteral(image), true);
    mzRDFStore.Assert(newNode, locationProp, RDF.GetLiteral(location), true);
    mzRDFStore.Assert(newNode, intervalProp, RDF.GetLiteral(interval), true);
    mzRDFStore.Assert(newNode, ordinalProp, RDF.GetLiteral(ordinal), true);

    mzRDFStore.Assert(newNode, pluginsProp, RDF.GetLiteral(plugins), true);
    mzRDFStore.Assert(newNode, javascriptProp, RDF.GetLiteral(javascript), true);
    mzRDFStore.Assert(newNode, metaredirectsProp, RDF.GetLiteral(metaredirects), true);
    mzRDFStore.Assert(newNode, subframesProp, RDF.GetLiteral(subframes), true);
    mzRDFStore.Assert(newNode, imagesProp, RDF.GetLiteral(images), true);

    mzRDFStore.Assert(newNode, selectedProp, RDF.GetLiteral(selected), true);
    tabContainer.AppendElement(newNode, true);
    mzRDFStore.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  }
  writePref("bool", "multizilla.tabs.startgroup", true);
}
////////////////////////////////////////////////////////////////////////////////
// Modified function, from a prior version, restores all tabs/windows
function restoreBrowserSession(aCheckFirstWindow)
{
  if ((!isFirstWindow() && aCheckFirstWindow) || !mzRDFStore)
    return;
  // We can only check for duplicates *after* the startup page(s) is/are loaded!
  if (readMyPref("multizilla.prevent.double", "bool", true)) {
    var delayed = false;
    var tab = null;
    if (pref.getIntPref("browser.startup.page") == 1) {
      var pageCount = pref.getIntPref("browser.startup.homepage.count");
      tab = gBrowser.mTabContainer.childNodes;
      if (pageCount != tab.length)
        delayed = true;
      for (var i = 0; i < tab.length; i++) {
        if (tab[i].hasAttribute("busy")) {
          delayed = true;
          break;
        }
      }
    }
    if (delayed && aCheckFirstWindow) {
      setTimeout(restoreBrowserSession, 1000, true);
      return;
    }
  }
  // Lets restore all tabs from a priorly saved browser session
  restoreTabsFromResource("urn:session-0:window-0:tabs:root");
}
//////////////////////////////////////////////////////////////////////////
function restoreTabsFromResource(aResource)
{
  var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
  var tabContainer = RDFC.MakeSeq(mzRDFStore, RDF.GetResource(aResource)).GetElements();
  var index = 0;

  while (tabContainer.hasMoreElements()) {
    try {
      var resource = tabContainer.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      var labels = mzRDFStore.ArcLabelsOut(resource);
      var dataCollection = new Array;
      var labelIndex = -1;
      while (labels.hasMoreElements()) {
        try {
          var label = labels.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
          var data = mzRDFStore.GetTarget(resource, label, true);
          var collection = {};
          collection.label = label.Value.replace(/.*#/, "") || "error";
          data = data.QueryInterface(Components.interfaces.nsIRDFLiteral);
          collection.value = data.Value;
          labelIndex++;
          dataCollection[labelIndex] = collection;
        } 
        catch(ex) { 
          dump("\nFailure in xxx to label/data:\n" + ex);
        }
	  }
      index++;
      if (labelIndex >= 0)
        addTabFromCollection(index, dataCollection);
    } catch(ex) {
      dump("\n"+ex);
      continue;
    }
  }
}
//////////////////////////////////////////////////////////////////////////
// This function opens a new tab, execept for the first URL, and updates 
// the corresponding browser properties for this tab. Some of the tabs 
// attributes are also changed, execept for the ordinal. We saved the 
// tabs in the right order, so there's no need to set the ordinal
function addTabFromCollection(aIndex, aDataCollection)
{
  var URL = aDataCollection[3].value;
  // We don't want to open useless blank tabs
  if (URL == "about:blank")
    return;
  var aTab = gBrowser.selectedTab;
  // TODO: use useFirstPage();
  if (aIndex > 1) // The first URL must be loaded in the blank tab
    aTab = gBrowser.addTab();
  // Get the browser for this (new) tab
  var browser = gBrowser.getBrowserForTab(aTab);
  // Modify browser properties before we load the URL for this tab
  browser.docShell.allowPlugins = (aDataCollection[6].value == "true") ? true : false;
  browser.docShell.allowJavascript = (aDataCollection[7].value == "true") ? true : false;
  browser.docShell.allowMetaRedirects = (aDataCollection[8].value == "true") ? true : false;
  browser.docShell.allowSubframes = (aDataCollection[9].value == "true") ? true : false;
  browser.docShell.allowImages = (aDataCollection[10].value == "true") ? true : false;
  // The properties are set so lets load the URL
  browser.loadURI(aDataCollection[3].value);
  // Do we need to set a reload interval?
  var interval = aDataCollection[4].value;
  // We only honor intervals of 30+ seconds
  if (interval >= 30000)
    gBrowser.setReloadInterval(aTab, interval);
  // Activate the (new) tab
  if (aDataCollection[11].value == "true")
    gBrowser.selectedTab = aTab;
}
//////////////////////////////////////////////////////////////////////////
// This function removes all given resources from multizilla.rdf
function removeAllResourcesFor(aResource)
{
  removeResources(aResource);
  removeDataResources(aResource);
  mzRDFStore.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
}
//////////////////////////////////////////////////////////////////////////
function removeResources(aResource)
{
  var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
  var tabContainer = RDFC.MakeSeq(mzRDFStore, RDF.GetResource(aResource)).GetElements();
  /*** 
  * EXAMPLE: removeResources('urn:session-0:window-0:tabs:tab-0');
  * removes:
    <RDF:Description about="urn:session-0:window-0:tabs:tab-0"
                     MZ:label="mozdev.org - multizilla: index"
                     MZ:custom="false"
                     MZ:image="http://www.mozilla.org/images/mozilla-16.png"
                     MZ:location="http://multizilla.mozdev.org/index.html"
                     MZ:interval="0"
                     MZ:ordinal="0"
                     MZ:plugins="true"
                     MZ:javascript="true"
                     MZ:metaredirects="true"
                     MZ:subframes="true"
                     MZ:images="true"
                     MZ:selected="false" /> */

  while (tabContainer.hasMoreElements()) {
    try {
      var resource = tabContainer.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      var labels = mzRDFStore.ArcLabelsOut(resource);

      while (labels.hasMoreElements()) {
        try {
          var label = labels.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
          var data = mzRDFStore.GetTarget(resource, label, true);
          mzRDFStore.Unassert(resource, label, data);
        } 
        catch(ex) { 
          // dump("\n'removeResources' failed in label/data:\n" + ex);
        }
	  }
    } catch(ex) {
      // dump("\n'removeResources' failed!\n"+ex);
      continue;
    }
  }
}
//////////////////////////////////////////////////////////////////////////
// The given resource will be removed from multizilla.rdf
function removeDataResources(aResource)
{
  var resource = RDF.GetResource(aResource);
  var labels = mzRDFStore.ArcLabelsOut(resource);
  /*** 
  * EXAMPLE: removeDataResources('urn:session-0:window-0:tabs:root');
  * removes:
    <RDF:Seq about="urn:session-0:window-0:tabs:root">
      <RDF:li resource="urn:session-0:window-0:tabs:tab-0"/>
    </RDF:Seq> */

  while (labels.hasMoreElements()) {
    try {
      var label = labels.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      var value = mzRDFStore.GetTarget(resource, label, true);
      mzRDFStore.Unassert(resource, label, value);
    }
    catch(ex) {
      // dump("\n'removeDataResources' failed!\n"+ex);
    }
  }
}

